'use strict';

var _redis = require('../utils/redis.js');

var _redis2 = _interopRequireDefault(_redis);

var _download = require('../utils/download.js');

var _download2 = _interopRequireDefault(_download);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Subscribe {
	constructor(wechatapi, config) {
		this.wechatapi = wechatapi;
		this.broadcastImgKey = "crm:broadcast:img";
		this.config = config;
		this.redis = _redis2.default.getCli(config);
	}
	subscribe(message) {
		let openid = message.to.id;
		console.log(message);
		return new Promise((resolve, reject) => {
			if (message.type == 'TEXT') {
				this.sendTEXT(openid, message).then(status => {
					resolve();
				}).catch(err => {
					reject(err);
				});
			} else if (message.type == 'IMAGE') {
				this.sendIMAGE(openid, message).then(status => {
					resolve();
				}).catch(err => {
					reject(err);
				});
			} else if (message.type == 'AUDIO') {
				this.sendAUDIO(openid, message).then(status => {
					resolve();
				}).catch(err => {
					reject(err);
				});
			} else if (message.type == 'EXTENSION') {
				this.sendEXTENSION(openid, message).then(status => {
					resolve();
				}).catch(err => {
					reject(err);
				});
			} else {
				reject("invlid msg type");
			}
		});
	}

	sendTEXT(openid, message) {
		return new Promise((resolve, reject) => {
			this.wechatapi.sendText(openid, message.content, (err, result) => {
				if (err) {
					reject(err);
				} else {
					resolve(result);
				}
			});
		});
	}

	sendIMAGE(openid, message) {
		return new Promise((resolve, reject) => {
			let broadcastRecordId = message.ext.broadcastRecordId;
			let isBroadcast = message.ext.isBroadcast;
			let url = message.attachmentUrl;
			if (isBroadcast) {
				this.redis.sismember(this.broadcastImgKey, broadcastRecordId, (err, media_id) => {
					if (media_id) {
						return this.wechatapi.sendIMAGE(openid, media_id);
					} else {
						return this.downloadAndSendImg(openid, message.ext.url, broadcastRecordId);
					}
				});
			} else {
				return this.downloadAndSendImg(openid, url, broadcastRecordId);
			}
		});
	}

	sendEXTENSION(openid, message) {
		return new Promise((resolve, reject) => {
			let ext = message.ext;
			if (ext && ext.type === 'ACTIVITY') {
				article = {
					title: ext.title,
					picurl: ext.img,
					description: ext.description,
					url: ext.url
				};
				wechatApi.sendNews(openid, [article], (err, result) => {
					if (err) {
						reject(err);
					} else {
						resolve(result);
					}
				});
			}
			if (ext && ext.type === 'ENDING') {
				wechatApi.sendText(openid, message.content, (err, result) => {
					if (err) {
						reject(err);
					} else {
						resolve(result);
					}
				});
			}
		});
	}

	sendAUDIO(openid, message) {
		return new Promise((resolve, reject) => {
			//todo
			resolve();
		});
	}
	downloadAndSendImg(openid, url, broadcastRecordId) {
		return new Promise((resolve, reject) => {
			(0, _download2.default)(url).then(file => {
				console.log(file);
				this.wechatapi.uploadImage(file, (err, data) => {
					if (err) {
						reject(err);
					} else {
						if (broadcastRecordId) {
							this.redis.sadd(this.broadcastImgKey, broadcastRecordId);
						}
						this.wechatapi.sendImage(openid, data.media_id, (err, data) => {
							if (err) {
								reject(err);
								console.log(err);
							} else {
								resolve(data);
								console.log(data);
							}
						});
					}
				});
			});
		});
	}
}

module.exports = Subscribe;